unit umain;
{
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/
 *
 * Copyright (C) 2013, Robert W.B. Linn, Pinneberg, Germany (www.rwblinn.de).
 *
 * See readme.txt for latest information.
}

(*
  // Einige SQL bespiele
  Var sSQL: String;
  // Tabelle Buecher erstellen
  sSQL := 'DROP TABLE If Exists Buecher';
  SQLConnection.Execute(sSQL, NIL);
  sSQL := 'CREATE TABLE "Buecher" (' +
          '"id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE,' +
          '"titel" TEXT NOT NULL,' +
          '"autor" TEXT,' +
          '"isbn" TEXT)';
  SQLConnection.Execute(sSQL, NIL);
  // Buecher hinzufgen
  sSQL := 'INSERT INTO Buecher (titel, autor, isbn) VALUES ("Titel 1", "Autor 1", "ISBN1") ';
  SQLConnection.Execute(sSQL, NIL);
  sSQL := 'INSERT INTO Buecher (titel, autor, isbn) VALUES ("Titel 2", "Autor 2", "ISBN2") ';
  SQLConnection.Execute(sSQL, NIL);
  sSQL := 'INSERT INTO Buecher (titel, autor, isbn) VALUES ("Titel 3", "Autor 3", "ISBN3") ';
  SQLConnection.Execute(sSQL, NIL);
  // Buchtitel aktualisieren
  sSQL := 'UPDATE Buecher SET Titel="Buchtitel 2", ISBN="ISBN 2-2" WHERE Titel="Titel 2"';
  SQLConnection.Execute(sSQL, NIL);
  // Alle Buecher zeigen. Ergebnis in TDataSet FDSSQLResults
  sSQL := 'SELECT * FROM Buecher';
  SQLConnection.Execute(sSQL, NIL, FDSSQLResults);
  // und weitere
  CREATE UNIQUE INDEX idx_isbn ON Buecher (isbn)
  DROP INDEX idx_isbn
  CREATE VIEW AllBuecher AS SELECT * FROM Buecher
  DROP VIEW AllBuecher
  // Spezielle Pragma Befehle
  PRAGMA Table_Info(Buecher);
*)

Interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Data.DB, Data.SqlExpr,
  System.Actions, Vcl.ActnList, Data.DbxSqlite, ShellAPI, System.StrUtils,
  Vcl.Grids, Vcl.Menus, Data.FMTBcd;

Const
  CVERSION    = 'Beispielanwendung zur Kurzanleitung Delphi SQLite - v20130820';
  CCOPYRIGHT  = 'Copyright (c) 2013, Robert W.B. Linn, Pinneberg, Germany. www.rwblinn.de';

Type
  TfrmMain = class(TForm)
    memoSQLStatement: TMemo;
    btnSQL: TButton;
    memoSQLOutput: TMemo;
    btnCopyright: TButton;
    SQLConnection: TSQLConnection;
    ActionList: TActionList;
    ActionSQLConnection: TAction;
    ActionSQLExecute: TAction;
    ActionSQLResults: TAction;
    sgSQLResults: TStringGrid;
    ActionSQLResultsToStringGrid: TAction;
    cbSQLStatementSamples: TComboBox;
    pmMain: TPopupMenu;
    ActionSQLSelectAllBooksToHTML: TAction;
    ActionSQLSelectAllBooksToHTML1: TMenuItem;
    ActionSQLiteDBSelectAndConnect: TAction;
    SQLiteDatenbankwhlenundverbinden1: TMenuItem;
    lblSQLiteDatabase: TLabel;
    N1: TMenuItem;
    ActionSQLExecuteLineByLineOption: TAction;
    SQLbefehlZeilefrZeileausfhren1: TMenuItem;
    N2: TMenuItem;
    ActionCopyright: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbSQLStatementSamplesChange(Sender: TObject);
    procedure ActionSQLConnectionExecute(Sender: TObject);
    procedure ActionSQLExecuteExecute(Sender: TObject);
    procedure ActionSQLResultsExecute(Sender: TObject);
    procedure ActionSQLResultsToStringGridExecute(Sender: TObject);
    procedure ActionSQLSelectAllBooksToHTMLExecute(Sender: TObject);
    procedure ActionSQLiteDBSelectAndConnectExecute(Sender: TObject);
    procedure ActionSQLExecuteLineByLineOptionExecute(Sender: TObject);
    procedure ActionCopyrightExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FDB: String;              // SQLite Datenbank mit Pfad
    FDSSQLResults: TDataSet;  // Ergebnis SQL Befehle select und pragma
    Function DatasetRecordCount(DS: TDataSet): Integer;
  end;

var
  frmMain: TfrmMain;

implementation

Const
  CCRLF = #13#10;
  CSQLDROPTABLEBuecher = 'DROP TABLE IF EXISTS Buecher;';
  CSQLCREATETABLEBuecher = 'CREATE TABLE Buecher (' +
          'BuchID INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE, ' +
          'Titel VARCHAR(100) NOT NULL, ' +
          'AutorID INTEGER, ' +
          'ISBN  VARCHAR(20));';
  CSQLDROPTABLEAUTOREN = 'DROP TABLE IF EXISTS Autoren;';
  CSQLCREATETABLEAUTOREN = 'CREATE TABLE Autoren (' +
          'AutorID INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE,' +
          'Name VARCHAR(40), ' +
          'Beschreibung VARCHAR(255), ' +
          'Homepage VARCHAR(100));';
  CSQLDROPTABLELOG = 'DROP TABLE IF EXISTS Log;';
  CSQLCREATETABLELOG = 'CREATE TABLE Log (' +
          'LogID INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE,'+
          'Datum VARCHAR(20) NOT NULL, '+
          'Info VARCHAR(255) NOT NULL);';
  CSQLDROPTRIGGERBI = 'DROP TRIGGER IF EXISTS Buecher_I;';
  CSQLCREATETRIGGERBI = 'CREATE TRIGGER Buecher_I AFTER INSERT ON Buecher ' + CCRLF +
                        'BEGIN'  + CCRLF +
                        ' INSERT INTO Log(Datum, Info) ' + CCRLF +
                        ' VALUES (DATETIME(''now''), ''Neu: '' || new.Titel);'  + CCRLF +
                        'END;';
  CSQLDROPTRIGGERBU = 'DROP TRIGGER IF EXISTS Buecher_U;';
  CSQLCREATETRIGGERBU = 'CREATE TRIGGER Buecher_U AFTER UPDATE ON Buecher '  + CCRLF +
                        'BEGIN'  + CCRLF +
                        ' INSERT INTO Log(Datum, Info) ' + CCRLF +
                        ' VALUES (DATETIME(''now''), ''Gendert: '' || old.Titel || '' auf '' || new.Titel);'  + CCRLF +
                        'END;';
  CSQLDROPTRIGGERBD = 'DROP TRIGGER IF EXISTS Buecher_D;';
  CSQLCREATETRIGGERBD = 'CREATE TRIGGER Buecher_D AFTER DELETE ON Buecher '  + CCRLF +
                        'BEGIN'  + CCRLF +
                        ' INSERT INTO Log(Datum, Info) ' + CCRLF +
                        ' VALUES (DATETIME(''now''), ''Gelscht: '' || old.Titel);'  + CCRLF +
                        'END;';

  // Buecher hinzufgen
  CSQLINSERTBUCH1 = 'INSERT INTO Buecher (BuchID, Titel, AutorID, ISBN) VALUES (NULL, "Titel 1", 1, "ISBN1");';
  CSQLINSERTBUCH2 = 'INSERT INTO Buecher (BuchID, Titel, AutorID, ISBN) VALUES (NULL, "Titel 2", 2, "ISBN2");';
  CSQLINSERTBUCH3 = 'INSERT INTO Buecher (BuchID, Titel, AutorID, ISBN) VALUES (NULL, "Titel 3", 1, "ISBN3");';
  // Autoren hinzufgen
  CSQLINSERTAUTOR1 = 'INSERT INTO Autoren (AutorID, Name, Beschreibung) VALUES (NULL, "Autor 1", "Beschreibung Autor 1");';
  CSQLINSERTAUTOR2 = 'INSERT INTO Autoren (AutorID, Name, Beschreibung) VALUES (NULL, "Autor 2", "Beschreibung Autor 2");';

{$R *.dfm}
//******************************************************************************
// FORM
//******************************************************************************
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Caption := CVERSION;
  // Stringgrid fr select Ergebnis nicht zeigen
  sgSQLResults.Visible := False;
  // SQLite Datenbank
  FDB := ExtractFilePath(Application.ExeName) + 'buecher.db';
  // SQL Verbindung mit der SQLite Datenbank herstellen
  ActionSQLConnectionExecute(Sender);
  // Datenbank Tabellen erstellen und erste Datenstze hinzufgen
  // Tabelle Buecher
  memoSQLStatement.Text := CSQLDROPTABLEBuecher;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETABLEBuecher;
  ActionSQLExecuteExecute(Sender);
  // Tabelle Autoren
  memoSQLStatement.Text := CSQLDROPTABLEAUTOREN;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETABLEAUTOREN;
  ActionSQLExecuteExecute(Sender);
  // Tabelle Log
  memoSQLStatement.Text := CSQLDROPTABLELOG;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETABLELOG;
  ActionSQLExecuteExecute(Sender);
  // Trigger
  memoSQLStatement.Text := CSQLDROPTRIGGERBI;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETRIGGERBI;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLDROPTRIGGERBU;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETRIGGERBU;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLDROPTRIGGERBD;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLCREATETRIGGERBD;
  ActionSQLExecuteExecute(Sender);
  // Daten einfgen
  memoSQLStatement.Text := CSQLINSERTBUCH1;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLINSERTBUCH2;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLINSERTBUCH3;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLINSERTAUTOR1;
  ActionSQLExecuteExecute(Sender);
  memoSQLStatement.Text := CSQLINSERTAUTOR2;
  ActionSQLExecuteExecute(Sender);
  memoSQLOutput.Text := 'SQLite Datenbank erstellt am ' + DateTimeToStr(Now) + #13#10 +
                        'Tabellen:' + #13#10 +
                        CSQLCREATETABLEBuecher + #13#10 +
                        CSQLCREATETABLEAutoren + #13#10 +
                        CSQLCREATETABLELog;
  memoSQLOutput.Text := StringReplace(memoSQLOutput.Text, 'CREATE TABLE ', '', [rfReplaceAll, rfIgnoreCase]);
  memoSQLOutput.Text := StringReplace(memoSQLOutput.Text, ');', '', [rfReplaceAll, rfIgnoreCase]);
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  With memoSQLStatement Do Begin
    SetFocus;
    Text := '<SQL Befehl>';
    SelStart:=0;
    SelLength:=Length(Text);
  End;
end;

//******************************************************************************
// ACTIONS
//******************************************************************************
procedure TfrmMain.ActionSQLiteDBSelectAndConnectExecute(Sender: TObject);
var
  bOK: Boolean;
begin
  bOK := InputQuery('SQLite Datenbank whlen und ffnen', 'SQLite Datenbank', FDB);
  If bOK Then
    ActionSQLConnectionExecute(Sender);
end;

procedure TfrmMain.ActionCopyrightExecute(Sender: TObject);
begin
  ShellExecute(Handle, nil, PChar('http://www.rwblinn.de'), nil, nil, SW_SHOW);
end;

procedure TfrmMain.ActionSQLConnectionExecute(Sender: TObject);
Var i : Integer;
begin
  sgSQLResults.Visible := False;
  Try
    With SQLConnection Do Begin
      Connected := False;
      Params.Clear;
      Params.Add('ColumnMetaDataSupported=True');
      Params.Add('FailIfMissing=False');
      Params.Add('Database=' + FDB);
      lblSQLiteDatabase.Caption := 'Datenbank: ' + FDB;
      Connected := True;
      memoSQLOutput.Text := 'SQLite Datenbank Verbindung hergestellt (' +
                            DateTimeToStr(Now) + ')' + #13#10 + 'Parameter:';
      For i := 0 To Params.Count - 1 Do
        memoSQLOutput.Lines.Add(Params[i]);
    End;
  Except
    on E: EDatabaseError do
      memoSQLOutput.Text := 'SQLConnection Fehler:' + #13#10 + E.Message;
  end;
end;

procedure TfrmMain.ActionSQLExecuteExecute(Sender: TObject);
Var nRT: Integer; // Resultat SQLConnection.Execute
    i: Integer;   // zeielncounter fr SQL Befehl LineByLine
Begin
  sgSQLResults.Visible := False;
  FDSSQLResults := NIL;
  If memoSQLStatement.Text = '' Then Exit;
  If NOT SQLConnection.Connected Then Exit;
  // SQL Befhel direkt ausfhren
  If NOT ActionSQLExecuteLineByLineOption.Checked Then Begin
    Try
      memoSQLOutput.Text := 'SQL Befehl ausgefhrt am ' + DateTimeToStr(Now);
      If (LeftStr(LowerCase(memoSQLStatement.Text), Length('select')) = 'select') Or
         (LeftStr(LowerCase(memoSQLStatement.Text), Length('pragma')) = 'pragma') Then Begin
        // SQL select query mit Ergebnis
        nRT := SQLConnection.Execute(memoSQLStatement.Text, NIL, FDSSQLResults);
      End
      Else Begin
        // SQL non select Befehl liefert kein Resultat, wie
        // create table, drop table, insert into, delete, update
        nRT := SQLConnection.ExecuteDirect(memoSQLStatement.Text);
      End;
      memoSQLOutput.Lines.Add('Anzahl Datenstze bearbeitet: ' + IntToStr(nRT));
    // Wurden Datenstze gefunden, dann anzeigen im Memo und ggf. Stringgrid
    If FDSSQLResults <> NIL Then Begin
      ActionSQLResultsExecute(Sender);
    End;
    Except
      on E: Exception do
        memoSQLOutput.Text := memoSQLStatement.Text + #13#10 + 'SQL Befehl Fehler: ' + #13#10 + E.Message;
    End;
  End;
  // SQL Befehl Line by Line ausfhren
  If ActionSQLExecuteLineByLineOption.Checked Then Begin
    memoSQLOutput.Lines.Clear;
    For i := 0 To memoSQLStatement.Lines.Count - 1 Do Begin
      If memoSQLStatement.Lines[i] <> '' Then Begin
        Try
          memoSQLOutput.Lines.Add('SQL Befehl ausgefhrt am ' + DateTimeToStr(Now));
          memoSQLOutput.Lines.Add('SQL: ' + memoSQLStatement.Lines[i]);
          If (LeftStr(LowerCase(memoSQLStatement.Lines[i]), Length('select')) = 'select') Or
              (LeftStr(LowerCase(memoSQLStatement.Lines[i]), Length('pragma')) = 'pragma') Then Begin
            // SQL select query mit Ergebnis
            nRT := SQLConnection.Execute(memoSQLStatement.Lines[i], NIL, FDSSQLResults);
          End
          Else Begin
            // SQL non select Befehl liefert kein Resultat, wie
            // create table, drop table, insert into, delete, update
            nRT := SQLConnection.ExecuteDirect(memoSQLStatement.Lines[i]);
          End;
          memoSQLOutput.Lines.Add('Anzahl Datenstze bearbeitet: ' + IntToStr(nRT));
          // Wurden Datenstze gefunden, dann anzeigen im Memo und ggf. Stringgrid
          If FDSSQLResults <> NIL Then
            ActionSQLResultsExecute(Sender);
        Except
          on E: Exception do
            memoSQLOutput.Text := 'SQL Befehl Fehler: ' + #13#10 + E.Message;
        End;
      End;
    End;
  End;
end;

procedure TfrmMain.ActionSQLExecuteLineByLineOptionExecute(Sender: TObject);
begin
  // ActionSQLExecuteLineByLineOption.Checked := NOT ActionSQLExecuteLineByLineOption.Checked;
end;

procedure TfrmMain.ActionSQLResultsExecute(Sender: TObject);
Var
  fSLNames: TStringList;  // Liste Feldnamen Tabelle
  i: Integer;             // Laufvariable
  currentField: TField;   // Aktuelle Feld
  currentLine: String;    // Aktuelle Memozeile mit Feldergebnis
  nCnt: Integer;          // Recordcounter
Begin
  sgSQLResults.Visible := False;
  If FDSSQLResults = NIL Then Exit;
  If FDSSQLResults.IsEmpty Then Exit;
  nCnt := DatasetRecordCount(FDSSQLResults);
  // Check ob SELECT oder PRAGMA Befehl
  // SELECT
  If (LeftStr(LowerCase(memoSQLStatement.Text), Length('select')) = 'select') Then Begin
    memoSQLOutput.Lines.Add('Anzahl Datenstze: ' + IntToStr(nCnt));
    fSLNames := TStringList.Create;         // Liste der Feldnamen initialisieren
    FDSSQLResults.GetFieldNames(fSLNames);  // Feldnamen lesen aus Ergebnisdataset
    FDSSQLResults.First;                    // Ergebnis Satz fr Satz lesen
    While NOT FDSSQLResults.Eof Do Begin
      currentLine := '';
      For i := 0 To fSLNames.Count - 1 Do Begin
        currentField := FDSSQLResults.FieldByName(fSLNames[i]);
        currentLine := currentLine + ' ' + currentField.AsString;
      End;
      memoSQLOutput.Lines.Add(currentLine); // Datensatz anzeigen
      FDSSQLResults.Next;
    End;
    fSLNames.Free;
    ActionSQLResultsToStringGridExecute(Sender);
  End;
  // PRAGMA
  If (LeftStr(LowerCase(memoSQLStatement.Text), Length('pragma')) = 'pragma') Then Begin
    memoSQLOutput.Lines.Add('(Spaltennr, Spaltenname, Data Type, NOT NULL, Vorgabewert)');
    FDSSQLResults.First;
    While Not FDSSQLResults.Eof Do Begin
      currentLine := '';
      // Hole alle Infos aus dem datensatz
      // Beispiel pragma table_info(Tabelle) liefert
      // column name, data type, whether or not the column can be NULL, and the default value for the column
      For i := 0 to FDSSQLResults.FieldCount-1 do
        currentLine := currentLine + ' ' + FDSSQLResults.Fields[i].AsString;
      memoSQLOutput.Lines.Add(currentLine); // Datensatz anzeigen
      FDSSQLResults.Next;
    End;
  End;
End;

procedure TfrmMain.ActionSQLResultsToStringGridExecute(Sender: TObject);
// Ergebnis SQL Select Befehl im StringGrid zeigen
Var
  nCol, nLin: Integer;
begin
  // Datenstze gefunden = Zeigen
  If Not FDSSQLResults.IsEmpty Then Begin
    sgSQLResults.Visible := True;
    sgSQLResults.ColCount := FDSSQLResults.FieldCount + 1;
    sgSQLResults.RowCount := DatasetRecordCount(FDSSQLResults) + 1;
    sgSQLResults.FixedCols := 1;
    sgSQLResults.FixedRows := 1;
    // Spaltenberschriften setzen
    For nCol := 0 To FDSSQLResults.FieldCount - 1 Do
      sgSQLResults.Cells[nCol + 1, 0] := FDSSQLResults.Fields[nCol].FieldName;
    nLin := 0;
    // Daten im Stringgrid zeigen
    FDSSQLResults.First;
    While Not FDSSQLResults.Eof Do Begin
      For nCol := 0 To FDSSQLResults.FieldCount - 1 Do Begin
        sgSQLResults.Cells[nCol + 1, nLin + sgSQLResults.FixedRows] :=
          FDSSQLResults.Fields[nCol].AsString;
      End;
      FDSSQLResults.Next;
      Inc(nLin);
    End;
  End;
end;

procedure TfrmMain.ActionSQLSelectAllBooksToHTMLExecute(Sender: TObject);
// Ergebnis SQL Select * From Buecher; in HTML Tabelle ausgeben und im
// Standardbrowser zeigen
// CREATE TABLE "Buecher" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE,"titel" TEXT NOT NULL,"autor" TEXT,"isbn" TEXT);
// CREATE UNIQUE INDEX idx_isbn ON Buecher (isbn);
Const
  CCRLF = #13#10;
  CHTMLCSS    = '<style type='''+ 'text/css' + '''>' + CCRLF +
                'table { width:100%; background-color:#D7E4F2; border:1px; } '+ CCRLF +
                'h1 { font:bold 2.0em Arial; color:#000080; background-color:#; border:1px; solid #000; vertical-align:top; overflow:hidden; }' + CCRLF +
                'th { font:bold 1.6em Arial; color:#000080; background-color:#F0F8FF; border:1px; solid #000; vertical-align:top; overflow:hidden; }' + CCRLF +
                'td { font:normal 1.0em Arial; color:#000000; background-color:#; border:1px; solid #000; vertical-align:top; overflow:hidden; }' + CCRLF +
                'div { font:normal 0.6em Arial; color:#000080; background-color:#; border:1px; solid #000; vertical-align:top; overflow:hidden; }' + CCRLF +
                '</style>';
  CHTMLBEGIN  = '<HTML>' + CCRLF + '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN">' + CCRLF +
                '<HEAD>' + CCRLF+ CHTMLCSS + CCRLF +
                '</HEAD><BODY bgcolor=#FFFFFF><H1>Liste der Buchtitel</H1><TABLE>';
  CHTMLEND    = '</TABLE><p><div>Erstellt mittels ropDelphiSQLite (c) 2013 Robert W.B. Linn, Pinnberg, Germany</div></p></BODY></HTML>';

Var DS: TDataSet;
    fSL: TStringList;
    sFile: String;
    i: Integer;
    sField: String;
Begin
  // SQL direkt ausfhren
  DS := NIL;
  Try
    SQLConnection.Execute('SELECT * FROM Buecher', NIL, DS);
  Except
    on E: Exception do
      memoSQLOutput.Text := 'SQL Befehl Fehler: ' + #13#10 + E.Message;
  End;
  With DS Do Begin
    If IsEmpty Then Exit;
    sFile := ExtractFilePath(Application.ExeName) + 'buecherliste.htm';
    fSL := TStringList.Create;
    fSL.Add(CHTMLBEGIN);
    // Spaltenberschrift - Feldnamen
    fSL.Add('<TR>');
    For i := 0 To FieldCount - 1 Do
      fSL.Add('<TH>' + Fields[i].FieldName + '</TH>');
    fSL.Add('</TR>');
    // Inhalt Tabelle ausgeben
    First;
    While Not Eof Do Begin
      fSL.Add('<TR>');
      For i := 0 To FieldCount - 1 Do Begin
        sField := Fields[i].AsString;
        If sField = '' Then sField := '&nbsp;';
        fSL.Add('<TD>' + sField + '</TD>');
      End;
      Next;
      fSL.Add('</TR>');
    End;
    fSL.Add(CHTMLEND);
    fSL.SaveToFile(sFile);
    ShellExecute(Handle, nil, PChar(sFile), nil, nil, SW_SHOW);
  End;
end;

procedure TfrmMain.cbSQLStatementSamplesChange(Sender: TObject);
begin
  memoSQLStatement.Text := cbSQLStatementSamples.Text;
end;

Function TfrmMain.DatasetRecordCount(DS: TDataSet): Integer;
// Anzahl Datenstze zurckgeben
Var i : Integer;
Begin
  Result := 0;
  If DS = NIL Then Exit;
  If DS.IsEmpty Then Exit;
  i := 0;
  DS.First;
  While Not DS.Eof Do Begin
    i := i + 1;
    DS.Next;
  End;
  Result := i;
End;


end.

