unit umain;
{
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/
 *
 * Copyright (C) 2013, Robert W.B. Linn, Pinneberg, Germany (www.rwblinn.de).
 *
}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Data.DB, Data.SqlExpr,
  Vcl.ExtCtrls, Data.DbxSqlite, Data.DbxCommon, Vcl.Imaging.pngimage, ShellAPI;

Const
  CVERSION = ' v20130820';
  CCOPYRIGHT = 'Copyright (c) 2013, Robert W.B. Linn, Pinneberg, Germany. www.rwblinn.de';

type
  TfrmMain = class(TForm)
    imageBlob: TImage;
    SQLConnection: TSQLConnection;
    btnLoadImages: TButton;
    lblTitle: TLabel;
    lbImages: TListBox;
    memoDescription: TMemo;
    lblCopyRight: TLabel;
    btnSaveImageToFile: TButton;
    lblDescription: TLabel;
    procedure btnLoadImagesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnClearImageClick(Sender: TObject);
    procedure lbImagesClick(Sender: TObject);
    procedure btnSaveImageToFileClick(Sender: TObject);
    procedure lblCopyRightClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FSQL : String;
    procedure InsertPicture(Sender: TObject; sTitle, sImagePath: String);
    procedure LoadPicture(Sender: TObject; sTitle: String);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
//
begin
  Caption := Caption + CVERSION;
  lblCopyright.Caption := CCOPYRIGHT;
  Try
    With SQLConnection Do Begin
      Connected := False;
      Params.Add('KeepConnection=True');
      Params.Add('FailIfMissing=False');
      Params.Add('Database=' + 'ropdelphisqliteblob.db');
      Connected := True;
      // Tabelle images erstellen - sicherstellen das tabelle auch gelscht wird
      FSQL := 'DROP TABLE IF EXISTS "images"';
      SQLConnection.ExecuteDirect(FSQL);
      FSQL := 'CREATE TABLE "images" (' +
          '"id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE,' +
          '"title" TEXT NOT NULL,' +
          '"image" BLOB NOT NULL)';
      SQLConnection.ExecuteDirect(FSQL);
    End;
    btnLoadImagesClick(Sender);
  Except
    on E: EDatabaseError do
      ShowMessage(E.Message);
  end;
end;

procedure TfrmMain.btnClearImageClick(Sender: TObject);
begin
  imageBlob.Picture := Nil;
end;

procedure TfrmMain.btnLoadImagesClick(Sender: TObject);
// Bilder in der SQL Tabelle "images" laden
Var DS: TDataSet;
Begin
  DS := NIL;
  InsertPicture(Sender, 'M', ExtractFilePath(Application.ExeName) + 'm.bmp');
  InsertPicture(Sender, 'O', ExtractFilePath(Application.ExeName) + 'o.bmp');
  InsertPicture(Sender, 'I', ExtractFilePath(Application.ExeName) + 'i.bmp');
  InsertPicture(Sender, 'N', ExtractFilePath(Application.ExeName) + 'n.bmp');
  // Listbox Titel anzeigen
  Try
    lbImages.Clear;
    FSQL := 'SELECT * FROM images;';
    SQLConnection.Execute(FSQL, NIL, DS);
    If DS = NIL Then Exit;
    DS.First;
    While NOT DS.Eof Do Begin
      lbImages.Items.Add(DS.FieldByName('title').AsString);
      DS.Next;
    End;
    lbImages.ItemIndex := 0;
    lbImagesClick(Sender);
  Except On E: Exception do
    ShowMessage(E.Message);
  End;
end;

procedure TfrmMain.btnSaveImageToFileClick(Sender: TObject);
// Image in eine Datei speichern
Var
  sFile: string;
  bOK: Boolean;
begin
  sFile := ExtractFilePath(Application.ExeName) + lblTitle.Caption + '.bmp'; //;
  bOK := InputQuery('Image speichern', 'Dateiname', sFile);
  If bOK Then Begin
    Try
      imageBlob.Picture.SaveToFile(sFile);
    Except
    on E: Exception do
      ShowMessage(E.Message);
    End;
  End;
end;

procedure TfrmMain.InsertPicture(Sender: TObject; sTitle, sImagePath: String);
// Titel und Image in der SQL Tabelle images laden
Var
  streamImage: TMemoryStream;     // Stream um Datei Image zu speichern
  LTransaction: TDBXTransaction;  // SQLConnection Transaktion
  LDBXCmd: TSQLQuery;             // SQL Query
  LParam: TParam;                 // Parameter fr SQL Query
  sSQL: String;                   // SQL Befehl
begin
  // SQL Insert Befehl zusammenstellen - achte auf "value ?" fr image
  sSQL := 'INSERT INTO images(title, image) VALUES("' + sTitle + '", ?)';
  // Stream erstellen - LoadFromFile kann noch erweitert werden mit z.B.
  // If FileExists...
  streamImage := TMemoryStream.Create;
  streamImage.LoadFromFile(sImagePath);
  // Transaktion erffnen
  LTransaction := SQLConnection.BeginTransaction;
  // Query Befehl zusammenstellen
  LDBXCmd := TSQLQuery.Create(SQLConnection);
  Try
    Try
      LDBXCmd.SQLConnection := SQLConnection;
      LDBXCmd.SQL.Text := sSQL;
      LParam := LDBXCmd.Params.CreateParam(ftBlob, 'Picture', ptInput);
      LParam.LoadFromStream(streamImage, ftBlob);
      LDBXCmd.ExecSQL;
    Except
      On E: Exception do
        SQLConnection.RollbackFreeAndNil(LTransaction);
    End;
    SQLConnection.CommitFreeAndNil(LTransaction);
  Finally
    LDBXCmd.Free;
  End;
End;

procedure TfrmMain.lbImagesClick(Sender: TObject);
// Imagelistbox Klick Event = zeigt Titel und Image im TLabel und TImage
begin
  With lbImages Do begin
    If ItemIndex <> - 1 Then Begin
      LoadPicture(Sender, Items[ItemIndex]);
    End;
  end;
end;

procedure TfrmMain.lblCopyRightClick(Sender: TObject);
begin
  ShellExecute(Handle, nil, PChar('http://www.rwblinn.de'), nil, nil, SW_SHOW);
end;

procedure TfrmMain.LoadPicture(Sender: TObject; sTitle: String);
// Titel und Bild in Label und Image laden
Var
  sSQL: String;       // SQL Befehl
  DS: TDataSet;       // Resultat SQL Select
  BlobField: TField;  // Blobfeld bentigt fr BlobStream
  BS: TStream;        // BS Blobstream
  BM: TBitMap;        // Bitmap im Image laden
Begin
  // Select Befehl nach Titel
  sSQL := 'SELECT * FROM images WHERE title = "' + sTitle + '"';
  Try
    // SQL Befehl ausfhren und Ergebnis in DS speichern
    DS := NIL;
    SQLConnection.Execute(sSQL, NIL, DS);
    // Ergebnis lesen und Inhalt Label, BlobField, Bitmap und Image zuweisen
    If DS <> NIL Then Begin
      lblTitle.Caption := DS.FieldByName('title').AsString;
      Blobfield := DS.FieldbyName('image');
      BS := DS.CreateBlobStream(BlobField, bmReadWrite);
      BM := TBitMap.Create;
      BM.LoadFromStream(BS);
      imageBlob.Picture.Bitmap := BM;
    End;
  Except
    on E: Exception do
      ShowMessage(E.Message);
  End;
End;

end.

