unit umain;
{
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/
 *
 * Copyright (C) 2013, Robert W.B. Linn, Pinneberg, Germany (www.rwblinn.de).
 *
}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.FMTBcd, Vcl.StdCtrls,
  System.Actions, Vcl.ActnList, Data.DB, Vcl.Grids, Vcl.DBGrids, Data.SqlExpr,
  Data.DbxSqlite, Vcl.ExtCtrls, Datasnap.Provider, Datasnap.DBClient,
  Vcl.DBCtrls, StrUtils;

Const
  CCRLF = #13#10;

Const
  CPRODUCTNAME    = 'SQLite Beispiel - DBGrid & DBNavigator';
  CVERSION        = 'v1.1 (Build: 20130831)';
  CAUTHORNAME     = 'Robert W.B. Linn, Pinneberg, Germany, www.rwblinn.de';

type
  TfrmMain = class(TForm)
    SQLConnection1: TSQLConnection;
    ActionList1: TActionList;
    pnlButtons: TPanel;
    btnDatabaseCreate: TButton;
    btnTableCreate: TButton;
    btnTableTestData: TButton;
    ActionDatabaseConnect: TAction;
    ActionTableCreate: TAction;
    ActionTableGetData: TAction;
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    ClientDataSet1: TClientDataSet;
    DataSetProvider1: TDataSetProvider;
    SQLDataSet1: TSQLDataSet;
    DBNavigator1: TDBNavigator;
    ActionTableInsertTestData: TAction;
    edSQLSelectStatement: TEdit;
    procedure ActionDatabaseConnectExecute(Sender: TObject);
    procedure ActionTableCreateExecute(Sender: TObject);
    procedure ActionTableGetDataExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
    procedure ClientDataSet1BeforeRefresh(DataSet: TDataSet);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionTableInsertTestDataExecute(Sender: TObject);
    procedure edSQLSelectStatementKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ClientDataSet1BeforePost(DataSet: TDataSet);
    Function  GetNextBuchID: Integer;
  private
    { Private declarations }
  public
    { Public declarations }
    Function DatasetRecordCount(DS: TDataSet): Integer;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Caption := CPRODUCTNAME + ' - ' + CVERSION;
  ActionDatabaseConnectExecute(Sender);
  //ActionTableCreateExecute(Sender);
  //ActionTableInsertTestDataExecute(Sender);
  ActionTableGetDataExecute(Sender);
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ClientDataSet1BeforeRefresh(ClientDataSet1);
end;

procedure TfrmMain.ActionDatabaseConnectExecute(Sender: TObject);
begin
  Try
    SQLDataset1.Close;
    SQLConnection1.Close;
    SQLConnection1.LoginPrompt := False;
    SQLConnection1.Params.Values['Database'] := ExtractFilePath(Application.ExeName) + 'buecher.db';
    SQLConnection1.Params.Values['FailIfMissing'] := 'False';
    SQLConnection1.Params.Values['RowsetSize'] := '100';
    SQLConnection1.Open;
    //ShowMessage('SQLite Database created: ' + SQLConnection1.Params.Values['Database'] );
  Except
    on E: EDatabaseError do
      ShowMessage('Fehler: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionTableCreateExecute(Sender: TObject);
// Create Table Buecher
begin
  Try
    SQLDataset1.Active := False;
    SQLDataset1.CommandText :=
      'DROP TABLE IF EXISTS Buecher';
    SQLDataset1.ExecSQL;
    // Note: Use VarChar to convert to stringfield, as using TEXT always result in a memofield.
    // If as memofield, then dbgrid cell entry displays (widememo)
    SQLDataset1.CommandText :=
      'CREATE TABLE Buecher (' +
        'BuchID INTEGER PRIMARY KEY AUTOINCREMENT,' +
        'Titel VarChar(20) NOT NULL, ' +
        'AutorID Integer,' +
        'ISBN VarChar(20))';
    SQLDataset1.ExecSQL;
    //ShowMessage('SQLite Table "Buecher" created.');
  Except
    on E: EDatabaseError do
      ShowMessage('Fehler: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionTableInsertTestDataExecute(Sender: TObject);
// Insert some testdate into table Buecher
Var i: Integer;
begin
  Try
    // OPTION1 : Using parameter
    // Insert 5 records
    For i := 1 To 5 Do Begin
      SQLDataset1.CommandText :=
      'INSERT OR REPLACE INTO Buecher (Titel, AutorID, ISBN) ' +
      'VALUES("Titel ' + IntToStr(i) + '", "' + IntToStr(i) + '", "' + IntToStr(i) + '");';
      SQLDataset1.ExecSQL;
    End;

    (*
    // OPTION 2: With parameter
    With SQLDataset1 Do Begin
      For i := 1 To 5 Do Begin
        // Parameter definieren (Spaltennamen verwenden)
        CommandText := 'INSERT INTO Buecher (BuchID, Titel, AutorID, ISBN) VALUES(:BuchID, :Titel, :AutorID, :ISBN);';
        // Parameter Werte zuweisen (Spaltennamen verwenden)
        ParamByName('BuchID').Value := NULL;
        ParamByName('Titel').Value := 'Mein Buch ' + IntToStr(i);
        ParamByName('AutorID').Value := i;
        ParamByName('ISBN').Value := 'ISBN' + IntToStr(i);
        // Und Werte in der Tabelle Buecher einfgen
        ExecSQL;
      End;
    End;
    *)
  Except
    on E: EDatabaseError do
      ShowMessage('Fehler: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionTableGetDataExecute(Sender: TObject);
// Retrieve data from sqlite table buecher and show in dbgrid
begin
  Try
    ClientDataSet1.Active := False;
    // Define the sql statement to select all Buecher - can be added with order statement etc.
    SQLDataset1.CommandText := 'SELECT * FROM Buecher;';
    (*
    SQLDataset1.CommandText := 'SELECT Buecher.BuchID,Buecher.Titel,Buecher.AutorID, Autoren.Name,Buecher.ISBN ' +
                               'FROM Autoren, Buecher ' +
                               'WHERE Autoren.AutorID=Buecher.AutorID';
    *)
    // Also as mentioned other selectionoptions
    //SQLDataset1.CommandText := 'SELECT * FROM Buecher WHERE id < 3;';
    SQLDataset1.Open;
    // Activate the clientdataset and show in the dbgrid
    ClientDataSet1.Active := True;
  Except
    on E: EDatabaseError do
      ShowMessage('Fehler: ' + E.Message);
  End;
end;

procedure TfrmMain.ClientDataSet1BeforePost(DataSet: TDataSet);
// Check if the autoincrementfield BuchID has value. If isnull, then set next BuchID
// This is required as the dataset always set fields to required, if these are not explocitly defined
// using the datasetproperty fielddefs
// If using the datasetproperty fielddefs with all fields defined, do set field BuchID required to false
// DataSet.FieldByName('BuchID').Required := False;
begin
  If TClientDataSet(DataSet).FieldByName('BuchID').IsNull Then
    TClientDataSet(DataSet).FieldByName('BuchID').Value := GetNextBuchID;
end;

procedure TfrmMain.ClientDataSet1BeforeRefresh(DataSet: TDataSet);
// Ensure to save the data from clientdataset1 in the sqlite database
begin
  If DataSet = Nil Then Abort;
  If TClientDataSet(DataSet).Active = False Then Abort;
  If TClientDataSet(DataSet).ChangeCount > 0 Then Begin
    Try
      TClientDataSet(DataSet).ApplyUpdates(-1);
    Except
      on E: EDatabaseError do
        // ShowMessage(E.Message);
    End;
  End;

end;

procedure TfrmMain.DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
  // NOT USED
begin
//
end;

procedure TfrmMain.edSQLSelectStatementKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  If Key = VK_RETURN Then Begin
    // showmessage(edSQLSelectStatement.Text);
    If Uppercase(LeftStr(edSQLSelectStatement.Text, Length('select'))) <> 'SELECT' Then
      Exit;
    Try
      ClientDataSet1.Active := False;
      SQLDataset1.Close;
      SQLDataset1.CommandText := edSQLSelectStatement.Text;
      //SQLDataset1.CommandText := 'SELECT * FROM Buecher WHERE BuchID > 10;';
      SQLDataset1.Open;
      // Activate clientdataset to display data in the dbgrid
      ClientDataSet1.Active := True;
    Except
      on E: EDatabaseError do
        ShowMessage('Fehler: ' + E.Message);
    End;
  End;
end;

Function TfrmMain.GetNextBuchID: Integer;
// Get the next autoincrement value for field BuchID
Begin
  SQLDataset1.Close;
  SQLDataset1.CommandText := 'SELECT MAX(BuchID) FROM Buecher;';
  SQLDataset1.Open;
  Result := SQLDataset1.Fields[0].AsInteger + 1;
  SQLDataset1.Close;
End;

Function TfrmMain.DatasetRecordCount(DS: TDataSet): Integer;
// Get recordcount using loop
Var i : Integer;
Begin
  Result := 0;
  If DS = NIL Then Exit;
  If DS.IsEmpty Then Exit;
  i := 0;
  DS.First;
  While Not DS.Eof Do Begin
    i := i + 1;
    DS.Next;
  End;
  Result := i;
End;

end.
