unit umain;
{
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/
 *
 * Copyright (C) 2013, Robert W.B. Linn, Pinneberg, Germany (www.rwblinn.de).
 *
}
(*
SELECT
Buecher.BuchID, Buecher.Titel, Buecher.AutorID, Autoren.Name, Buecher.ISBN
FROM
Autoren, Buecher
WHERE
Autoren.AutorID = Buecher.AutorID
ORDER
BY Buecher.Titel;

*)

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Variants, System.Classes, System.StrUtils,
  Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DbxSqlite, Data.FMTBcd, Data.DB,
  Data.SqlExpr, Vcl.Grids, Vcl.DBGrids, Datasnap.DBClient, Datasnap.Provider,
  Vcl.ExtCtrls, Vcl.ComCtrls, Vcl.StdCtrls, System.Actions, Vcl.ActnList;

Const
  CTESTDB = 'buecher.db';

type
  TfrmMain = class(TForm)
    SQLConnection: TSQLConnection;
    SQLDataSet: TSQLDataSet;
    DataSetProvider: TDataSetProvider;
    ClientDataSet: TClientDataSet;
    DataSource: TDataSource;
    pnlTestDBToDBGrid: TPanel;
    memoSQLiteDBQueryResult: TMemo;
    SQLQuery1: TSQLQuery;
    ActionList1: TActionList;
    pnlTop: TPanel;
    edSQLiteDBPath: TEdit;
    btnSQLiteDBOpen: TButton;
    lblAlerts: TLabel;
    pnlSQLiteDBQuery: TPanel;
    memoSQLiteDBQuery: TMemo;
    btnSQLiteDBQuery: TButton;
    OpenDialog1: TOpenDialog;
    ActionSQLiteDBConnect: TAction;
    ActionSQLiteDBTableList: TAction;
    ActionSQLiteDBOpen: TAction;
    ActionSQLiteQueryToMemo: TAction;
    ActionSQLiteQueryToDBGrid: TAction;
    dbgSQLiteDBQueryToDBGrid: TDBGrid;
    ActionSQLiteQueryToStringGrid: TAction;
    sgSQLiteDBQueryToStringGrid: TStringGrid;
    btnSQLiteDBQueryToDBGrid: TButton;
    btnSQLiteDBQueryToStringGrid: TButton;
    ActionSQLiteCreateDBandTables: TAction;
    Button1: TButton;
    gbDBTableList: TGroupBox;
    lbDBTableList: TListBox;
    ActionSQLiteQuery: TAction;
    procedure FormCreate(Sender: TObject);
    procedure ActionSQLiteDBConnectExecute(Sender: TObject);
    procedure ActionSQLiteDBTableListExecute(Sender: TObject);
    procedure ActionSQLiteDBOpenExecute(Sender: TObject);
    procedure ActionSQLiteQueryToMemoExecute(Sender: TObject);
    procedure ActionSQLiteQueryToDBGridExecute(Sender: TObject);
    procedure ActionSQLiteQueryToStringGridExecute(Sender: TObject);
    procedure ActionSQLiteCreateDBandTablesExecute(Sender: TObject);
    procedure lbDBTableListClick(Sender: TObject);
    procedure ActionSQLiteQueryExecute(Sender: TObject);
    //procedure SQLiteQueryDBQueryToStringGrid(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FSQLCommandText: String;
    FSQLDataSet: TDataSet;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

//******************************************************************************
// FORM
//******************************************************************************

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Deactivate all connections
  SQLConnection.Connected := False;
  SQLDataSet.Active       := SQLConnection.Connected;
  ClientDataSet.Active    := SQLConnection.Connected;
  // Init input & output fields
  edSQLiteDBPath.Text := ExtractFilePath(Application.ExeName) + CTESTDB;
  lblAlerts.Caption := 'Datenbank ist nicht geffnet.';
  memoSQLiteDBQuery.Lines.Text := '';
end;

procedure TfrmMain.lbDBTableListClick(Sender: TObject);
begin
  If lbDBTableList.ItemIndex = -1 Then Exit;
  memoSQLiteDBQuery.Lines.Text := 'select * from ' + lbDBTableList.Items[lbDBTableList.ItemIndex];
  ActionSQLiteQueryExecute(Sender);
end;

//******************************************************************************
// ACTIONS
//******************************************************************************
procedure TfrmMain.ActionSQLiteDBConnectExecute(Sender: TObject);
// SQLite DB connect
begin
  lblAlerts.Caption := 'Datenbank ist nicht geffnet.';
  // Checks
  If Not FileExists(edSQLiteDBPath.Text) Then Begin
    ShowMessage('Fehler SQLite Datenbankverbindung: Datenbank nicht gefunden!' + #13#10#13#10 +
                edSQLiteDBPath.Text);
    Exit;
  End;
  Try
    SQLConnection.Connected := False;
    SQLDataSet.Active := SQLConnection.Connected;
    // Parameter setzen
    SQLConnection.Params.Values['Database']       := edSQLiteDBPath.Text;
    SQLConnection.Params.Values['FailIfMissing']  := 'False';
    SQLConnection.Connected := True;
    lblAlerts.Caption := 'Datenbank ist geffnet.';
  Except On E: EDatabaseError Do
    ShowMessage('Fehler SQLite Datenbankverbindung: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionSQLiteDBOpenExecute(Sender: TObject);
// SQLite DB select and connect
begin
  OpenDialog1.Filename := edSQLiteDBPath.Text;
  If NOT OpenDialog1.Execute Then Exit;
  edSQLiteDBPath.Text := ExpandFileName(OpenDialog1.Filename);
  ActionSQLiteDBConnectExecute(Sender);
  ActionSQLiteDBTableListExecute(Sender);
end;

procedure TfrmMain.ActionSQLiteDBTableListExecute(Sender: TObject);
//
Var i: Integer;
    sLine: String;
begin
  // Check the connection with the SQLiteDB
  If SQLConnection.Connected = False Then ActionSQLiteDBConnectExecute(Sender);
  // If still not connected then exit - the error message will be given by the action
  If SQLConnection.Connected = False Then Exit;
  Try
    // memoSQLiteDBAnalysisResult.Lines.Add(SQLConnection.Params.Text);
    // Get the DB Information
    // FSQLCommandText := 'select * from sqlite_master;';
    // Get the tables
    lbDBTableList.Items.Clear;
    FSQLCommandText := 'SELECT name FROM SQLITE_MASTER WHERE TYPE="table" ORDER BY name;';
    SQLConnection.Execute(FSQLCommandText, NIL, FSQLDataSet);
    FSQLDataSet.First;
    While NOT FSQLDataSet.EOF Do Begin
      sLine := '';
      For i := 0 To FSQLDataSet.FieldCount - 1 Do Begin
        lbDBTableList.Items.Add(FSQLDataSet.Fields[i].AsString);
      End;
      FSQLDataSet.Next;
    End;
  Except // Fehler abfangen
    On E: EDatabaseError Do
      ShowMessage('Fehler SQLite Datenbankverbindung: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionSQLiteQueryToMemoExecute(Sender: TObject);
Var i: Integer;
    sLine: String;
begin
  FSQLCommandText := memoSQLiteDBQuery.Lines.Text;
  // SQLite DB verbinden
  If SQLConnection.Connected = False Then ActionSQLiteDBConnectExecute(Sender);
  If SQLConnection.Connected = False Then Exit;
  // Query ausfhren
  SQLConnection.Execute(FSQLCommandText, NIL, FSQLDataSet);
  //
  memoSQLiteDBQueryResult.Lines.Add('Anzahl Datenstze: ' + IntToStr(FSQLDataSet.RecordCount));
  memoSQLiteDBQueryResult.Lines.Add('----------');
  // Spaltennamen
  sLine := '';
  For i := 0 To FSQLDataSet.FieldCount - 1 Do
    sLine := sLine + ' ' + FSQLDataSet.Fields[i].FieldName;
  memoSQLiteDBQueryResult.Lines.Add(sLine);
  memoSQLiteDBQueryResult.Lines.Add('----------');
  FSQLDataSet.First;
  While NOT FSQLDataSet.EOF Do Begin
    sLine := '';
    For i := 0 To FSQLDataSet.FieldCount - 1 Do
      sLine := sLine + ' ' + FSQLDataSet.Fields[i].AsString;
    memoSQLiteDBQueryResult.Lines.Add(sLine);
    FSQLDataSet.Next;
  End;
  memoSQLiteDBQueryResult.SelStart := 0;
  memoSQLiteDBQueryResult.SelLength := 0;
end;

procedure TfrmMain.ActionSQLiteQueryExecute(Sender: TObject);
begin
  ActionSQLiteQueryToDBGridExecute(Sender);
  ActionSQLiteQueryToStringGridExecute(Sender);
  ActionSQLiteQueryToMemoExecute(Sender);
end;

procedure TfrmMain.ActionSQLiteQueryToDBGridExecute(Sender: TObject);
// Query to DBGRID --- only for select statements
begin
  memoSQLiteDBQueryResult.Lines.Clear;
  FSQLCommandText := memoSQLiteDBQuery.Text;
  // Check SQL query if has select tag
  If AnsiCompareText(LeftStr(FSQLCommandText, Length('select')), 'select') <> 0 Then Begin
    ShowMessage('Abbruch: SQL Query enthlt kein SELECT Befehl!');
    Exit;
  End;
  (*
  If (LeftStr(LowerCase(FSQLCommandText), Length('select')) <> 'select') Then Begin
    ShowMessage('Abbruch: SQL Query enthlt kein SELECT Befehl!');
    Exit;
  End;
  *)
  // SQLite DB verbinden
  If SQLConnection.Connected = False Then ActionSQLiteDBConnectExecute(Sender);
  If SQLConnection.Connected = False Then Exit;
  Try
    // ClientDataSet and SQLDataSet deactivate --- do not forget
    ClientDataSet.Active := False;
    SQLDataSet.Active := False;
    // Set SQL Statement
    SQLDataSet.CommandText := FSQLCommandText;
    // Activate
    SQLDataSet.Active := True;
    ClientDataSet.Active := True;
    // Run Query
    SQLDataSet.Open;
    // Show result --- count only
    // memoSQLiteDBQueryResult.Lines.Add('Anzahl Datenstze: ' + IntToStr(SQLDataSet.RecordCount));
  Except
    // Fehler abfangen
    On E: EDatabaseError Do
      ShowMessage('Fehler SQLite Datenbankverbindung: ' + E.Message);
  End;
end;

procedure TfrmMain.ActionSQLiteQueryToStringGridExecute(Sender: TObject);
// Query to STRINGGRID --- only for select statements
Var
  nCol, nLin: Integer;
begin
  memoSQLiteDBQueryResult.Lines.Clear;
  FSQLCommandText := memoSQLiteDBQuery.Text;
  // Check SQL query if has select tag
  If (LeftStr(LowerCase(FSQLCommandText), Length('select')) <> 'select') Then Begin
    ShowMessage('Abbruch: SQL Query enthlt kein SELECT Befehl!');
    Exit;
  End;
  // SQLite DB verbinden
  If SQLConnection.Connected = False Then ActionSQLiteDBConnectExecute(Sender);
  If SQLConnection.Connected = False Then Exit;
  // Query ausfhren
  SQLConnection.Execute(FSQLCommandText, NIL, FSQLDataSet);
  // Datenstze lesen und im StringGrid anzeigen
  Try
    If FSQLDataSet.IsEmpty Then Exit;
    // memoSQLiteDBQueryResult.Lines.Add('Anzahl Datenstze: ' + IntToStr(FSQLDataSet.RecordCount));
    sgSQLiteDBQueryToStringGrid.ColCount := FSQLDataSet.FieldCount + 1;
    sgSQLiteDBQueryToStringGrid.RowCount := FSQLDataSet.RecordCount + 1;
    sgSQLiteDBQueryToStringGrid.FixedCols := 1;
    sgSQLiteDBQueryToStringGrid.FixedRows := 1;
    sgSQLiteDBQueryToStringGrid.ColWidths[0] := 16;
    // Spaltenberschriften setzen
    For nCol := 0 To FSQLDataSet.FieldCount - 1 Do
      sgSQLiteDBQueryToStringGrid.Cells[nCol + 1, 0] := FSQLDataSet.Fields[nCol].FieldName;
    nLin := 0;
    // Daten im Stringgrid zeigen
    FSQLDataSet.First;
    While Not FSQLDataSet.Eof Do Begin
      For nCol := 0 To FSQLDataSet.FieldCount - 1 Do
        sgSQLiteDBQueryToStringGrid.Cells[nCol + 1, nLin + sgSQLiteDBQueryToStringGrid.FixedRows] := FSQLDataSet.Fields[nCol].AsString;
      FSQLDataSet.Next;
      Inc(nLin);
    End;

  Except On E: EDatabaseError Do
    ShowMessage('Fehler: ' + E.Message);
  End;
End;

procedure TfrmMain.ActionSQLiteCreateDBandTablesExecute(Sender: TObject);
Var sStr: String;
    i: Integer;
begin
  If FileExists(edSQLiteDBPath.Text) Then
    DeleteFile(edSQLiteDBPath.Text);
  Try
    // Create the Database
    SQLConnection.Connected := False;
    // Parameter setzen
    SQLConnection.Params.Values['Database']       := edSQLiteDBPath.Text;
    SQLConnection.Params.Values['FailIfMissing']  := 'False';
    SQLConnection.Connected := True;
    lblAlerts.Caption := 'Datenbank ist geffnet.';
    // Create Table
    FSQLCommandText := 'CREATE TABLE TBuecher (BuchID INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE, Titel VARCHAR(100) NOT NULL, AutorID INTEGER, ISBN VARCHAR(20));';
    SQLConnection.ExecuteDirect(FSQLCommandText);
    // Some Testdata
    FSQLCommandText := 'INSERT INTO TBuecher (BuchID,Titel,AutorID,ISBN) VALUES (NULL,"Schwere Wetter",1,"978-3897059207");';
    SQLConnection.ExecuteDirect(FSQLCommandText);
    FSQLCommandText := 'INSERT INTO TBuecher (BuchID,Titel,AutorID,ISBN) VALUES (NULL,"Nebelfront",1,"978-3954510269");';
    SQLConnection.ExecuteDirect(FSQLCommandText);
    // Show the Data
    FSQLCommandText := 'SELECT * FROM TBuecher;';
    SQLConnection.Execute(FSQLCommandText, NIL, FSQLDataSet);
    FSQLDataSet.First;
    While NOT FSQLDataSet.EOF Do Begin
      sStr := '';
      For i := 0 To FSQLDataSet.FieldCount - 1 Do
        sStr := sStr + ' ' + FSQLDataSet.Fields[i].AsString;
      ShowMessage(sStr);
      FSQLDataSet.Next;
    End;
  Except On E: EDatabaseError Do
    ShowMessage('Fehler SQLite Datenbankverbindung: ' + E.Message);
  End;
end;

end.

